import os.path
import os
from skconfig import Configuration, ConfigurationLocatorInfo
import shutil


# ------------------------------------------------------------------------------
#           override_yaml_base_folder
# ------------------------------------------------------------------------------
def override_yaml_base_folder(yamlfolder: str):
    """
    Overrides the folder location of the YAML files used for ALTIUS USASK yaml database.
    """
    os.environ['ALTIUS_L1PP_USASK_YAML_FOLDER'] = yamlfolder


# ------------------------------------------------------------------------------
#           check_specs_exist_in_user_space
# ------------------------------------------------------------------------------
def check_specs_exist_in_user_space():

    file_locator = ConfigurationLocatorInfo(packagename='altius_l1pp_preflight', groupname='altius_l1pp/usask-arg', yamlname='optics.yaml', location='user')
    try:
        config = Configuration(file_locator)
    except Exception:
        basedir, fname = os.path.split(__file__)
        sourcefolder = os.path.join(basedir, 'defaults')
        config = Configuration()
        filename = config.locatorinfo_to_filename(file_locator)
        destfolder, fname = os.path.split(filename)

        if not os.path.exists(destfolder):
            print('No user configuration found for altius_l1pp_preflight. Copying default configuration from {:s} to {:s}'.format(sourcefolder, destfolder))
            shutil.copytree(sourcefolder, destfolder)


# ------------------------------------------------------------------------------
#           load_specs
# ------------------------------------------------------------------------------
def load_specs(yamlname: str) -> Configuration:

    config = None
    special_location = os.environ.get('ALTIUS_L1PP_USASK_YAML_FOLDER')
    if (special_location is not None):
        try:
            config = Configuration(os.path.join(special_location, yamlname))
        except Exception:
            raise Exception('Error loading ALTIUS YAML configuration file [{}] from folder. Environment variable ALTIUS_L1PP_USASK_YAML_FOLDER is defined as [{:s}]. Check file exists and is a valid YAML file'.format(yamlname, special_location))
    else:
        file_locations = ('user', 'python')
        for file_location in file_locations:
            location = ConfigurationLocatorInfo(packagename='altius_l1pp_preflight', groupname='altius_l1pp/usask-arg', yamlname=yamlname, location=file_location)
            try:
                config = Configuration(location)
            except Exception:
                pass
            else:
                break

        if (config is None) or (config.values is None) or (len(config.values)) == 0:
            folder, aname = os.path.split(__file__)
            filename = os.path.normpath(os.path.join(folder, 'defaults', yamlname))
            config = Configuration(filename)
    assert (config.values is not None) and (len(config.values) > 0)
    return config
