import unittest
import os
import os.path
import numpy as np
import netCDF4
from altius_l1pp_preflight.psf.make_convolution_matrices import make_infield_convolution_matrix


# ------------------------------------------------------------------------------
#           AltiusL1ppTestConvolution
# ------------------------------------------------------------------------------
class AltiusL1ppTestConvolution(unittest.TestCase):
    '''
    These are unit tests which test the convolution software. This takes around 10 minutes to execute.
    '''

    def test_make_convolution(self):

        basedir, name = os.path.split(__file__)
        fname = os.path.join(basedir, 'deconv300.txt')
        goodanswers = np.loadtxt(fname)
        make_infield_convolution_matrix('uv', '2020-10-20', 300.0, filename='psf300.nc', overwrite=True)
        with netCDF4.Dataset('psf300.nc') as d:
            matrix = np.array(d['deconvolution_matrix'])
            newanswers = matrix[:, 400]
        diff = (newanswers - goodanswers) / goodanswers
        self.assertAlmostEqual(np.max(np.abs(diff)), 0.0, 5)
        os.unlink('psf300.nc')


if __name__ == "__main__":
    tests = AltiusL1ppTestConvolution()
    unittest.main()
